@@ -3,8 +3,9 @@ from __future__ import annotations
 import re
 import struct
 import sys
+from codecs import getincrementaldecoder
 from collections.abc import Callable, Mapping, Sequence
-from datetime import datetime, timedelta, timezone
+from datetime import date, datetime, timedelta, timezone
 from io import BytesIO
 from typing import TYPE_CHECKING, Any, TypeVar, cast, overload
 
@@ -29,9 +30,9 @@ if TYPE_CHECKING:
 T = TypeVar("T")
 
 timestamp_re = re.compile(
-    r"^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)"
-    r"(?:\.(\d{1,6})\d*)?(?:Z|([+-])(\d\d):(\d\d))$"
+    r"^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)" r"(?:\.(\d{1,6})\d*)?(?:Z|([+-])(\d\d):(\d\d))$"
 )
+incremental_utf8_decoder = getincrementaldecoder("utf-8")
 
 
 class CBORDecoder:
@@ -140,9 +141,7 @@ class CBORDecoder:
         return self._object_hook
 
     @object_hook.setter
-    def object_hook(
-        self, value: Callable[[CBORDecoder, Mapping[Any, Any]], Any] | None
-    ) -> None:
+    def object_hook(self, value: Callable[[CBORDecoder, Mapping[Any, Any]], Any] | None) -> None:
         if value is None or callable(value):
             self._object_hook = value
         else:
@@ -255,14 +254,10 @@ class CBORDecoder:
         ...
 
     @overload
-    def _decode_length(
-        self, subtype: int, allow_indefinite: Literal[True]
-    ) -> int | None:
+    def _decode_length(self, subtype: int, allow_indefinite: Literal[True]) -> int | None:
         ...
 
-    def _decode_length(
-        self, subtype: int, allow_indefinite: bool = False
-    ) -> int | None:
+    def _decode_length(self, subtype: int, allow_indefinite: bool = False) -> int | None:
         if subtype < 24:
             return subtype
         elif subtype == 24:
@@ -276,9 +271,7 @@ class CBORDecoder:
         elif subtype == 31 and allow_indefinite:
             return None
         else:
-            raise CBORDecodeValueError(
-                "unknown unsigned integer subtype 0x%x" % subtype
-            )
+            raise CBORDecodeValueError("unknown unsigned integer subtype 0x%x" % subtype)
 
     def decode_uint(self, subtype: int) -> int:
         # Major tag 0
@@ -303,8 +296,7 @@ class CBORDecoder:
                     length = self._decode_length(initial_byte & 0x1F)
                     if length is None or length > sys.maxsize:
                         raise CBORDecodeValueError(
-                            "invalid length for indefinite bytestring chunk 0x%x"
-                            % length
+                            "invalid length for indefinite bytestring chunk 0x%x" % length
                         )
                     value = self.read(length)
                     buf.append(value)
@@ -314,11 +306,20 @@ class CBORDecoder:
                     )
         else:
             if length > sys.maxsize:
-                raise CBORDecodeValueError(
-                    "invalid length for bytestring 0x%x" % length
-                )
+                raise CBORDecodeValueError("invalid length for bytestring 0x%x" % length)
+            elif length <= 65536:
+                result = self.read(length)
+            else:
+                # Read large bytestrings 65536 (2 ** 16) bytes at a time
+                left = length
+                buffer = bytearray()
+                while left:
+                    chunk_size = min(left, 65536)
+                    buffer.extend(self.read(chunk_size))
+                    left -= chunk_size
+
+                result = bytes(buffer)
 
-            result = self.read(length)
             self._stringref_namespace_add(result, length)
 
         return self.set_shareable(result)
@@ -354,17 +355,38 @@ class CBORDecoder:
                             "invalid length for indefinite string chunk 0x%x" % length
                         )
 
-                    value = self.read(length).decode("utf-8", self._str_errors)
+                    try:
+                        value = self.read(length).decode("utf-8", self._str_errors)
+                    except UnicodeDecodeError as exc:
+                        raise CBORDecodeValueError("error decoding unicode string") from exc
+
                     buf.append(value)
                 else:
-                    raise CBORDecodeValueError(
-                        "non-string found in indefinite length string"
-                    )
+                    raise CBORDecodeValueError("non-string found in indefinite length string")
         else:
             if length > sys.maxsize:
                 raise CBORDecodeValueError("invalid length for string 0x%x" % length)
 
-            result = self.read(length).decode("utf-8", self._str_errors)
+            if length <= 65536:
+                try:
+                    result = self.read(length).decode("utf-8", self._str_errors)
+                except UnicodeDecodeError as exc:
+                    raise CBORDecodeValueError("error decoding unicode string") from exc
+            else:
+                # Read and decode large text strings 65536 (2 ** 16) bytes at a time
+                codec = incremental_utf8_decoder(self._str_errors)
+                left = length
+                result = ""
+                while left:
+                    chunk_size = min(left, 65536)
+                    final = left <= chunk_size
+                    try:
+                        result += codec.decode(self.read(chunk_size), final)
+                    except UnicodeDecodeError as exc:
+                        raise CBORDecodeValueError("error decoding unicode string") from exc
+
+                    left -= chunk_size
+
             self._stringref_namespace_add(result, length)
 
         return self.set_shareable(result)
@@ -462,6 +484,15 @@ class CBORDecoder:
     #
     # Semantic decoders (major tag 6)
     #
+    def decode_epoch_date(self) -> date:
+        # Semantic tag 100
+        value = self._decode()
+        return self.set_shareable(date.fromordinal(value + 719163))
+
+    def decode_date_string(self) -> date:
+        # Semantic tag 1004
+        value = self._decode()
+        return self.set_shareable(date.fromisoformat(value))
 
     def decode_datetime_string(self) -> datetime:
         # Semantic tag 0
@@ -514,7 +545,13 @@ class CBORDecoder:
     def decode_epoch_datetime(self) -> datetime:
         # Semantic tag 1
         value = self._decode()
-        return self.set_shareable(datetime.fromtimestamp(value, timezone.utc))
+
+        try:
+            tmp = datetime.fromtimestamp(value, timezone.utc)
+        except (OverflowError, OSError, ValueError) as exc:
+            raise CBORDecodeValueError("error decoding datetime from epoch") from exc
+
+        return self.set_shareable(tmp)
 
     def decode_positive_bignum(self) -> int:
         # Semantic tag 2
@@ -584,9 +621,7 @@ class CBORDecoder:
             raise CBORDecodeValueError("shared reference %d not found" % value)
 
         if shared is None:
-            raise CBORDecodeValueError(
-                "shared value %d has not been initialized" % value
-            )
+            raise CBORDecodeValueError("shared value %d has not been initialized" % value)
         else:
             return shared
 
@@ -594,23 +629,49 @@ class CBORDecoder:
         # Semantic tag 30
         from fractions import Fraction
 
-        return self.set_shareable(Fraction(*self._decode()))
+        inputval = self._decode(immutable=True, unshared=True)
+        try:
+            value = Fraction(*inputval)
+        except (TypeError, ZeroDivisionError) as exc:
+            if not isinstance(inputval, tuple):
+                raise CBORDecodeValueError(
+                    "error decoding rational: input value was not a tuple"
+                ) from None
+
+            raise CBORDecodeValueError("error decoding rational") from exc
+
+        return self.set_shareable(value)
 
     def decode_regexp(self) -> re.Pattern[str]:
         # Semantic tag 35
-        return self.set_shareable(re.compile(self._decode()))
+        try:
+            value = re.compile(self._decode())
+        except re.error as exc:
+            raise CBORDecodeValueError("error decoding regular expression") from exc
+
+        return self.set_shareable(value)
 
     def decode_mime(self) -> Message:
         # Semantic tag 36
         from email.parser import Parser
 
-        return self.set_shareable(Parser().parsestr(self._decode()))
+        try:
+            value = Parser().parsestr(self._decode())
+        except TypeError as exc:
+            raise CBORDecodeValueError("error decoding MIME message") from exc
+
+        return self.set_shareable(value)
 
     def decode_uuid(self) -> UUID:
         # Semantic tag 37
         from uuid import UUID
 
-        return self.set_shareable(UUID(bytes=self._decode()))
+        try:
+            value = UUID(bytes=self._decode())
+        except (TypeError, ValueError) as exc:
+            raise CBORDecodeValueError("error decoding UUID value") from exc
+
+        return self.set_shareable(value)
 
     def decode_stringref_namespace(self) -> Any:
         # Semantic tag 256
@@ -715,10 +776,12 @@ semantic_decoders: dict[int, Callable[[CBORDecoder], Any]] = {
     35: CBORDecoder.decode_regexp,
     36: CBORDecoder.decode_mime,
     37: CBORDecoder.decode_uuid,
+    100: CBORDecoder.decode_epoch_date,
     256: CBORDecoder.decode_stringref_namespace,
     258: CBORDecoder.decode_set,
     260: CBORDecoder.decode_ipaddress,
     261: CBORDecoder.decode_ipnetwork,
+    1004: CBORDecoder.decode_date_string,
     55799: CBORDecoder.decode_self_describe_cbor,
 }
 
